==> AnnBuilder_1.3.3.tar.gz: AnnBuilder/R/map2LL.R <==
map2LL <- function(pkgPath, organism, version, author,
              url = "ftp://ftp.ncbi.nih.gov/refseq/LocusLink/"){
    # Write AnnInfo to .GlobalEnv to make Annotation information available
    makeSrcInfo()
    pkgName <- paste(organism, "LLMappings", sep = "")
    # Create a data package with no data
    createEmptyDPkg(pkgName, pkgPath, force = TRUE)
    # Write the template for man pages
    writeManPage(pkgName, pkgPath, "#REPLACEME#", "",
                         src = get("#REPLACEME#", AnnInfo)$src)
    # Read data with mappings between LL and UG
    ug <-  read.table(paste(url, getExten("ug"), sep = ""), sep = "\t",
                        header = FALSE, as.is = TRUE)
    # Retain values for the organism
    ug <- ug[grep(getOrgName(organism, "short"), ug[,2]), ]
    ll2UG <- mergeRowByKey(ug, keyCol = 1)
    saveData2Env(ll2UG, splitEntry, pkgName, pkgPath, "LL2UG")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                              paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "LL2UG.Rd", sep = "")),
                   list(REPLACEME = "LL2UG", KEY = "LocusLink id",
                        VALUE = "UniGene id",
                        OTHERS = ""), "#")
    ug2LL <- mergeRowByKey(ug, keyCol = 2)
    saveData2Env(ug2LL, splitEntry, pkgName, pkgPath, "UG2LL")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                              paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "UG2LL.Rd", sep = "")),
                   list(REPLACEME = "UG2LL", VALUE = "LocusLink id",
                        KEY = "UniGene id",
                        OTHERS = ""), "#")
    # Read data with mappings between LL and GO. This is only good for
    # mappings with no evidence code
    # go <- read.table(paste(url, getExten("go"), sep = ""), sep = "\t",
    #                    header = FALSE, as.is = TRUE)
    # Retain those for the organism
    # go <- go[is.element(go[,1], ug[,1]),]
    # Retain those for the organism
    # ll2GO <- mergeRowByKey(go, keyCol = 1)
    # saveData2Env(ll2GO, splitEntry, pkgName, pkgPath, "LL2GO")
    # copySubstitute(file.path(pkgPath, pkgName, "man",
    #                       paste(pkgName, "#REPLACEME#.Rd", sep = "")),
    #                       file.path(pkgPath, pkgName,
    #                       "man", paste(pkgName, "LL2GO.Rd", sep = "")),
    #               list(REPLACEME = "LL2GO", KEY = "LocusLink id",
    #                    VALUE = "Gene Ontology id"), "#")
    # go2LL <- mergeRowByKey(go, keyCol = 2)
    # saveData2Env(go2LL, splitEntry, pkgName, pkgPath, "GO2LL")
    # copySubstitute(file.path(pkgPath, pkgName, "man",
    #                       paste(pkgName, "#REPLACEME#.Rd", sep = "")),
    #                       file.path(pkgPath, pkgName,
    #                       "man", paste(pkgName, "GO2LL.Rd", sep = "")),
    #               list(REPLACEME = "GO2LL", VALUE = "LocusLink id",
    #               KEY = "Gene Ontology id"), "#")

    # Parse LL_tmpl data to get the mappings between LL and PMID
    oriParser <- file.path(.path.package("AnnBuilder"), "data",
                            "llMappingParser")
    newParser <- file.path(.path.package("AnnBuilder"), "temp",
                           "llMappingParser")
    # Write organism to the parser to be used
    copySubstitute(oriParser, newParser,
                   list(REPLACEME = getOrgName(organism, "scientific")), "#")
    ll <- LL(srcUrl = paste(url, getExten("ll"), sep = ""),
                       parser = newParser)
    # llMapping contains three columns - locusid, pmid, and go
    options(show.error.messages = FALSE)
    llMapping <- try(parseData(ll, ncol = 3, fromWeb = TRUE))
    options(show.error.messages = TRUE)
    if(inherits(llMapping, "try-error")){
        stop(paste("Failed to get or parse LocusLink data becaus of:\n\n",
                   llMapping))
    }
    # Do pmid first
    saveData2Env(llMapping[, c(1, 2)], pkgName = pkgName,
                 pkgPath = pkgPath, envName = "LL2PMID")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                             paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                   file.path(pkgPath, pkgName, "man",
                             paste(pkgName, "LL2PMID.Rd", sep = "")),
                   list(REPLACEME = "LL2PMID",
                        KEY = "LocusLink id", VALUE = "PubMed id",
                        OTHERS = ""), "#")
    # Now do the reverse mapping for PMID
    pmid2LL <- getReverseMapping(llMapping[, c(1,2)])
    saveData2Env(pmid2LL, pkgName = pkgName,
                         pkgPath = pkgPath, envName = "PMID2LL")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                             paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                   file.path(pkgPath, pkgName, "man",
                             paste(pkgName, "PMID2LL.Rd", sep = "")),
                   list(REPLACEME = "PMID2LL",
                        KEY = "PubMed id", VALUE = "LocusLink id",
                        OTHERS = ""), "#")
    # Do GO ids
    saveData2Env(llMapping[, c(1, 3)], pkgName = pkgName, fun = twoStepSplit,
                 pkgPath = pkgPath, envName = "LL2GO")
    other <- paste(" Names of values are evidence code. Detailed ",
                   "descriptions of evidence code are available at ",
                   "\\\\url{www.geneontology.org/doc/GO.evidence.html}.",
                   sep  = "")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                           paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "LL2GO.Rd", sep = "")),
                   list(REPLACEME = "LL2GO", KEY = "LocusLink id",
                        VALUE = "Gene Ontology id",
                        OTHERS = other), "#")
    # Do the reverse mapping for go
    go2LL <- reverseMap4GO(llMapping[, c(1, 3)], sep = ";")
    saveData2Env(go2LL, twoStepSplit, pkgName, pkgPath, "GO2LL")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                           paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "GO2LL.Rd", sep = "")),
                   list(REPLACEME = "GO2LL", VALUE = "LocusLink id",
                        KEY = "Gene Ontology id",
                        OTHERS = other), "#")
    # Write man pages and so on
    writeDescription(pkgName, pkgPath, version, author, dataSrc = "NCBI")
    writeFun (pkgPath, pkgName, organism = "")
    writeMan4Fun(pkgName, pkgPath)
    # Write .First.lib
    writeZZZ(pkgPath, pkgName)
    # Write the quality control data
    # Quality control
    getDPStats("", pkgName, pkgPath)
    writeMan4QC(pkgName, pkgPath)
    file.remove(file.path(pkgPath, pkgName, "man",
                           paste(pkgName, "#REPLACEME#.Rd", sep = "")))

    return(invisible())
}

getUrl4Org <- function(organism){
    switch(toupper(organism),
           HUMAN = return(),
           MOUSE = return(),
           RAT = return(),
           stop("Organism not supported"))
}

getOrgName <- function(organism, what = c("scientific", "short")){
    what <- tolower(match.arg(what))
    names <- cbind(c("human", "mouse", "rat", "fly"),
                   c("Homo sapiens", "Mus musculus", "Rattus norvegicus",
                     "Drosophila melanogaster"), c("Hs", "Mm", "Rn", "Dm"))
    if(what == "scientific"){
        return(names[names[,1] == organism, 2])
    }else{
        return(names[names[,1] == organism, 3])
    }
}

getFullIDName <- function(ID){
    switch(toupper(ID),
           LL = return("LocusLink id"),
           UG = return("UniGene id"),
           GO = return("Gene Ontology id"),
           PMID = return("PubMid id"),
           stop("Unknown id name"))
}

# Just do not what to throw the code away
.keepAround <- function(){
    # Read data with mappings between LL and GG
    go <- read.table(paste(url, getExten("go"), sep = ""), sep = "\t",
                        header = FALSE, as.is = TRUE)
    ll2GO <- mergeRowByKey(go, keyCol = 1)
    saveColSepData(ll2GO, pkgName, pkgPath, "LL2GO")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                           paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "LL2GO.Rd", sep = "")),
                   list(REPLACEME = "LL2GO", KEY = "LocusLink id",
                        VALUE = "Gene Ontology id"), "#")
    go2LL <- mergeRowByKey(go, keyCol = 2)
    saveColSepData(go2LL, pkgName, pkgPath, "GO2LL")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                           paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "GO2LL.Rd", sep = "")),
                   list(REPLACEME = "GO2LL", VALUE = "LocusLink id",
                        KEY = "Gene Ontology id"), "#")
    # Read data with mappings between LL and RefSeq
    ref <-  read.table(paste(url, getExten("ref"), sep = ""), sep = "\t",
                        header = FALSE, as.is = TRUE)
    ll2Ref <- mergeRowByKey(cbind(ref[,1], gsub("(^.*)\\..*", "\\1",
                                                   ref[,2])), keyCol = 1)
    saveColSepData(ll2Ref, pkgName, pkgPath, "LL2REF")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                              paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "LL2REF.Rd", sep = "")),
                   list(REPLACEME = "LL2REF", KEY = "LocusLink id",
                        VALUE = "RefSeq id"), "#")
    ref2LL <- mergeRowByKey(cbind(ref[,1], gsub("(^.*)\\..*", "\\1",
                                                   ref[,2])), keyCol = 2)
    saveColSepData(ref2LL, pkgName, pkgPath, "REF2LL")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                              paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "REF2LL.Rd", sep = "")),
                   list(REPLACEME = "REF2LL", VALUE = "LocusLink id",
                        KEY = "RefSeq id"), "#")
    # Read data with mappings between LL and UG
    ug <-  read.table(paste(url, getExten("ug"), sep = ""), sep = "\t",
                        header = FALSE, as.is = TRUE)
    ll2UG <- mergeRowByKey(ug, keyCol = 1)
    saveColSepData(ll2UG, pkgName, pkgPath, "LL2UG")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                              paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "LL2UG.Rd", sep = "")),
                   list(REPLACEME = "LL2UG", KEY = "LocusLink id",
                        VALUE = "UniGene id"), "#")
    ug2LL <- mergeRowByKey(ug, keyCol = 2)
    saveColSepData(ug2LL, pkgName, pkgPath, "UG2LL")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                              paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "UG2LL.Rd", sep = "")),
                   list(REPLACEME = "UG2LL", VALUE = "LocusLink id",
                        KEY = "UniGene id"), "#")
    # Write man pages and so on
    writeDescription(pkgName, pkgPath, version, author)
    writeFun (pkgPath, pkgName, organism = "")
    writeMan4Fun(pkgName, pkgPath)
    # Write .First.lib
    writeZZZ(pkgPath, pkgName)
    # Write the quality control data
    # Quality control
    getDPStats("", pkgName, pkgPath)
    writeMan4QC(pkgName, pkgPath)
    file.remove(file.path(pkgPath, pkgName, "man",
                           paste(pkgName, "#REPLACEME#.Rd", sep = "")))
}

getExten <- function(what){

    switch(toupper(what),
           "GO" = return("loc2go"),
           "REF" = return("loc2ref"),
           "UG" = return("loc2UG"),
           "LL" = return("LL_tmpl.gz"),
           stop("Source type not supported"))
}

saveData2Env <- function(data, fun = splitEntry, pkgName, pkgPath, envName){
    env <- new.env(hash = TRUE, parent = NULL)
    if(nrow(data) != 0){
        multiassign(data[,1], lapply(data[,2], fun), env)
    }
    assign(paste(pkgName, envName, sep = ""), env)
    save(list = paste(pkgName, envName, sep = ""),
         file = file.path(pkgPath, pkgName, "data",
         paste(pkgName, envName, ".rda", sep = "")))
}


getReverseMapping <- function(data, sep = ";"){
    if(nrow(data) == 0){
        return(data)
    }
    seped <- sapply(data[,2], splitEntry)
    reps <- sapply(seped, length)
    data <- cbind(rep(data[,1], reps), unlist(seped, use.names = FALSE))
    data <- mergeRowByKey(data, keyCol = 2)
    return(data)
}

reverseMap4GO <- function(data, sep = ";"){
    appendEC <- function(entry){
        ids <- unlist(strsplit(entry[2], ";"))
        return(paste(paste(ids, gsub("GO:[0-9]*", "", entry[1]),
                           sep = ""), sep = "", collapse = ";"))
    }
    if(nrow(data) == 0){
        return(data)
    }
    # Get reverse mappings first
    reversed <- getReverseMapping(data, sep)
    # Append "@evidence code" to the end of each mapped values
    reversed[,2] <- apply(reversed, 1, appendEC)

    # Remove evidence code from GO ids
    reversed[,1] <- gsub("@.*", "", reversed[,1])

    return(mergeRowByKey(reversed, keyCol = 1))
}

==> AnnBuilder_1.4.2.tar.gz: AnnBuilder/R/map2LL.R <==
map2LL <- function(pkgPath, organism, version, author,
                   url = "ftp://ftp.ncbi.nih.gov/refseq/LocusLink/"){
    # Write AnnInfo to .GlobalEnv to make Annotation information available
    makeSrcInfo()
    pkgName <- paste(organism, "LLMappings", sep = "")
    # Create a data package with no data
    createEmptyDPkg(pkgName, pkgPath, force = TRUE)
    # Write the template for man pages
    writeManPage(pkgName, pkgPath, "#REPLACEME#", "",
                         src = get("#REPLACEME#", AnnInfo)$src)
    # Read data with mappings between LL and UG
    ug <-  read.table(paste(url, getExten("ug"), sep = ""), sep = "\t",
                        header = FALSE, as.is = TRUE)
    # Retain values for the organism
    ug <- ug[grep(getOrgName(organism, "short"), ug[,2]), ]
    ll2UG <- mergeRowByKey(ug, keyCol = 1)
    saveData2Env(ll2UG, splitEntry, pkgName, pkgPath, "LL2UG")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                              paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "LL2UG.Rd", sep = "")),
                   list(REPLACEME = "LL2UG", KEY = "LocusLink id",
                        VALUE = "UniGene id",
                        OTHERS = ""), "#")
    ug2LL <- mergeRowByKey(ug, keyCol = 2)
    saveData2Env(ug2LL, splitEntry, pkgName, pkgPath, "UG2LL")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                              paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "UG2LL.Rd", sep = "")),
                   list(REPLACEME = "UG2LL", VALUE = "LocusLink id",
                        KEY = "UniGene id",
                        OTHERS = ""), "#")
    # Parse LL_tmpl data to get the mappings between LL and PMID
    oriParser <- file.path(.path.package("AnnBuilder"), "scripts",
                            "llMappingParser")
    newParser <- file.path(.path.package("AnnBuilder"), "temp",
                           "llMappingParser")
    # Write organism to the parser to be used
    copySubstitute(oriParser, newParser,
                   list(REPLACEME = getOrgName(organism, "scientific")), "#")
    ll <- LL(srcUrl = paste(url, getExten("ll"), sep = ""),
                       parser = newParser)
    # llMapping contains three columns - locusid, pmid, and go
    options(show.error.messages = FALSE)
    llMapping <- try(parseData(ll, ncol = 3, fromWeb = TRUE))
    options(show.error.messages = TRUE)
    if(inherits(llMapping, "try-error")){
        stop(paste("Failed to get or parse LocusLink data becaus of:\n\n",
                   llMapping))
    }
    # Do pmid first
    saveData2Env(llMapping[, c(1, 2)], pkgName = pkgName,
                 pkgPath = pkgPath, envName = "LL2PMID")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                             paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                   file.path(pkgPath, pkgName, "man",
                             paste(pkgName, "LL2PMID.Rd", sep = "")),
                   list(REPLACEME = "LL2PMID",
                        KEY = "LocusLink id", VALUE = "PubMed id",
                        OTHERS = ""), "#")
    # Now do the reverse mapping for PMID
    pmid2LL <- getReverseMapping(llMapping[, c(1,2)])
    saveData2Env(pmid2LL, pkgName = pkgName,
                         pkgPath = pkgPath, envName = "PMID2LL")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                             paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                   file.path(pkgPath, pkgName, "man",
                             paste(pkgName, "PMID2LL.Rd", sep = "")),
                   list(REPLACEME = "PMID2LL",
                        KEY = "PubMed id", VALUE = "LocusLink id",
                        OTHERS = ""), "#")
    # Do GO ids
    saveData2Env(llMapping[, c(1, 3)], pkgName = pkgName, fun = twoStepSplit,
                 pkgPath = pkgPath, envName = "LL2GO")
    other <- paste(" Names of values are evidence code. Detailed ",
                   "descriptions of evidence code are available at ",
                   "\\\\url{www.geneontology.org/doc/GO.evidence.html}.",
                   sep  = "")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                           paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "LL2GO.Rd", sep = "")),
                   list(REPLACEME = "LL2GO", KEY = "LocusLink id",
                        VALUE = "Gene Ontology id",
                        OTHERS = other), "#")
    # Do the reverse mapping for go
    go2LL <- reverseMap4GO(llMapping[, c(1, 3)], sep = ";")
    saveData2Env(go2LL, twoStepSplit, pkgName, pkgPath, "GO2LL")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                           paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "GO2LL.Rd", sep = "")),
                   list(REPLACEME = "GO2LL", VALUE = "LocusLink id",
                        KEY = "Gene Ontology id",
                        OTHERS = other), "#")
    #go <- GO(srcUrl = getSrcUrl("GO"))
    #goData <- readData(go, xml = TRUE, fromWeb = TRUE)
    #go2All <- mapGO2AllProbe(go2LL, goData, "", sep = ";", all = TRUE)
    #saveData2Env(go2All, twoStepSplit, pkgName, pkgPath, "GO2ALL")
    #other <- paste(" Names of values are evidence code. Detailed ",
    #               "descriptions of evidence code are available at ",
    #               "\\\\url{www.geneontology.org/doc/GO.evidence.html}.",
    #               sep  = "")
    #copySubstitute(file.path(pkgPath, pkgName, "man",
    #                       paste(pkgName, "#REPLACEME#.Rd", sep = "")),
    #                       file.path(pkgPath, pkgName,
    #                       "man", paste(pkgName, "GO2ALL.Rd", sep = "")),
    #               list(REPLACEME = "GO2ALL", VALUE = "LocusLink id",
    #                    KEY = "Gene Ontology id",
    #                    OTHERS = other), "#")
    # Write man pages and so on
    writeDescription(pkgName, pkgPath, version, author, dataSrc = "NCBI")
    writeFun (pkgPath, pkgName, organism = "")
    writeMan4Fun(pkgName, pkgPath)
    # Write .First.lib
    writeZZZ(pkgPath, pkgName)
    # Write the quality control data
    # Quality control
    getDPStats("", pkgName, pkgPath)
    writeMan4QC(pkgName, pkgPath)
    file.remove(file.path(pkgPath, pkgName, "man",
                           paste(pkgName, "#REPLACEME#.Rd", sep = "")))

    return(invisible())
}

getUrl4Org <- function(organism){
    switch(toupper(organism),
           HUMAN = return(),
           MOUSE = return(),
           RAT = return(),
           stop("Organism not supported"))
}

getOrgName <- function(organism, what = c("scientific", "short")){
    what <- tolower(match.arg(what))
    names <- cbind(c("human", "mouse", "rat", "fly"),
                   c("Homo sapiens", "Mus musculus", "Rattus norvegicus",
                     "Drosophila melanogaster"), c("Hs", "Mm", "Rn", "Dm"))
    if(what == "scientific"){
        return(names[names[,1] == organism, 2])
    }else{
        return(names[names[,1] == organism, 3])
    }
}

getFullIDName <- function(ID){
    switch(toupper(ID),
           LL = return("LocusLink id"),
           UG = return("UniGene id"),
           GO = return("Gene Ontology id"),
           PMID = return("PubMid id"),
           stop("Unknown id name"))
}

# Just do not what to throw the code away
.keepAround <- function(){
    # Read data with mappings between LL and GG
    go <- read.table(paste(url, getExten("go"), sep = ""), sep = "\t",
                        header = FALSE, as.is = TRUE)
    ll2GO <- mergeRowByKey(go, keyCol = 1)
    saveColSepData(ll2GO, pkgName, pkgPath, "LL2GO")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                           paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "LL2GO.Rd", sep = "")),
                   list(REPLACEME = "LL2GO", KEY = "LocusLink id",
                        VALUE = "Gene Ontology id"), "#")
    go2LL <- mergeRowByKey(go, keyCol = 2)
    saveColSepData(go2LL, pkgName, pkgPath, "GO2LL")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                           paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "GO2LL.Rd", sep = "")),
                   list(REPLACEME = "GO2LL", VALUE = "LocusLink id",
                        KEY = "Gene Ontology id"), "#")
    # Read data with mappings between LL and RefSeq
    ref <-  read.table(paste(url, getExten("ref"), sep = ""), sep = "\t",
                        header = FALSE, as.is = TRUE)
    ll2Ref <- mergeRowByKey(cbind(ref[,1], gsub("(^.*)\\..*", "\\1",
                                                   ref[,2])), keyCol = 1)
    saveColSepData(ll2Ref, pkgName, pkgPath, "LL2REF")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                              paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "LL2REF.Rd", sep = "")),
                   list(REPLACEME = "LL2REF", KEY = "LocusLink id",
                        VALUE = "RefSeq id"), "#")
    ref2LL <- mergeRowByKey(cbind(ref[,1], gsub("(^.*)\\..*", "\\1",
                                                   ref[,2])), keyCol = 2)
    saveColSepData(ref2LL, pkgName, pkgPath, "REF2LL")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                              paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "REF2LL.Rd", sep = "")),
                   list(REPLACEME = "REF2LL", VALUE = "LocusLink id",
                        KEY = "RefSeq id"), "#")
    # Read data with mappings between LL and UG
    ug <-  read.table(paste(url, getExten("ug"), sep = ""), sep = "\t",
                        header = FALSE, as.is = TRUE)
    ll2UG <- mergeRowByKey(ug, keyCol = 1)
    saveColSepData(ll2UG, pkgName, pkgPath, "LL2UG")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                              paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "LL2UG.Rd", sep = "")),
                   list(REPLACEME = "LL2UG", KEY = "LocusLink id",
                        VALUE = "UniGene id"), "#")
    ug2LL <- mergeRowByKey(ug, keyCol = 2)
    saveColSepData(ug2LL, pkgName, pkgPath, "UG2LL")
    copySubstitute(file.path(pkgPath, pkgName, "man",
                              paste(pkgName, "#REPLACEME#.Rd", sep = "")),
                           file.path(pkgPath, pkgName,
                           "man", paste(pkgName, "UG2LL.Rd", sep = "")),
                   list(REPLACEME = "UG2LL", VALUE = "LocusLink id",
                        KEY = "UniGene id"), "#")
    # Write man pages and so on
    writeDescription(pkgName, pkgPath, version, author)
    writeFun (pkgPath, pkgName, organism = "")
    writeMan4Fun(pkgName, pkgPath)
    # Write .First.lib
    writeZZZ(pkgPath, pkgName)
    # Write the quality control data
    # Quality control
    getDPStats("", pkgName, pkgPath)
    writeMan4QC(pkgName, pkgPath)
    file.remove(file.path(pkgPath, pkgName, "man",
                           paste(pkgName, "#REPLACEME#.Rd", sep = "")))
}

getExten <- function(what){

    switch(toupper(what),
           "GO" = return("loc2go"),
           "REF" = return("loc2ref"),
           "UG" = return("loc2UG"),
           "LL" = return("LL_tmpl.gz"),
           stop("Source type not supported"))
}

saveData2Env <- function(data, fun = splitEntry, pkgName, pkgPath, envName){
    env <- new.env(hash = TRUE, parent = NULL)
    if(nrow(data) != 0){
        multiassign(data[,1], lapply(data[,2], fun), env)
    }
    assign(paste(pkgName, envName, sep = ""), env)
    save(list = paste(pkgName, envName, sep = ""),
         file = file.path(pkgPath, pkgName, "data",
         paste(pkgName, envName, ".rda", sep = "")))
}


getReverseMapping <- function(data, sep = ";"){
    if(nrow(data) == 0){
        return(data)
    }
    seped <- sapply(data[,2], splitEntry)
    reps <- sapply(seped, length)
    data <- cbind(rep(data[,1], reps), unlist(seped, use.names = FALSE))
    data <- mergeRowByKey(data, keyCol = 2)
    return(data)
}

reverseMap4GO <- function(data, sep = ";", type = c("ll2GO", "GO2LL")){
    type <- match.arg(type)

    appendEC <- function(entry){
        ids <- unlist(strsplit(entry[2], ";"))
        if(type == "ll2GO"){
            return(paste(paste(ids, gsub("GO:[0-9]*", "", entry[1]),
                           sep = ""), sep = "", collapse = ";"))
        }else{
            return(paste(paste(ids, gsub("[0-9]*", "", entry[1]),
                           sep = ""), sep = "", collapse = ";"))
        }
    }
    if(nrow(data) == 0){
        if(length(data) != 2){
            return(data)
        }else{
            data <- cbind(data[2], data[1])
        }
    }
    # Get reverse mappings first
    reversed <- getReverseMapping(data, sep)
    # Append "@evidence code" to the end of each mapped values
    reversed[,2] <- apply(reversed, 1, appendEC)

    # Remove evidence code from GO ids
    reversed[,1] <- gsub("@.*", "", reversed[,1])

    return(mergeRowByKey(reversed, keyCol = 1))
}
